@@ -6,6 +6,14 @@ r"""
 Changes
 =======
 
+0.8.1 released on 2009-04-30
+----------------------------
+
+ * Fix memory leak due to circular references and __del__. [Reported by Valentine Gogichashvili]
+ * Fix encoding normalization. [Reported by Marc Silver]
+ * Fix "method" decorator to return callable when val is None.
+ * Fix startup of clusters in silent mode.
+
 0.8.0 released on 2009-04-03
 ----------------------------
 
