@@ -25,16 +25,23 @@ class test_cluster(unittest.TestCase):
 
 	def start_cluster(self):
 		self.cluster.start(logfile = None)
-		self.cluster.wait_until_started(timeout = 30)
+		self.cluster.wait_until_started(timeout = 10)
 
 	def init(self, *args, **kw):
 		self.cluster.init(*args, **kw)
 		self.cluster.settings.update({
 			'max_connections' : '8',
 			'listen_addresses' : 'localhost',
-			'port' : '6543'
+			'port' : '6543',
+			'silent_mode' : 'off',
 		})
 
+	def testSilentMode(self):
+		self.init(logfile = None)
+		self.cluster.settings['silent_mode'] = 'on'
+		# if it fails to start(ClusterError), silent_mode is not working properly.
+		self.start_cluster()
+
 	def testSuperPassword(self):
 		self.init(
 			user = 'test',
